@@ -83,9 +83,7 @@ static void iwd_shutdown(void)
 		return;
 	}
 
-#ifdef HAVE_DBUS
 	dbus_shutdown();
-#endif
 	netdev_shutdown();
 
 	timeout = l_timeout_create(1, main_loop_quit, NULL, NULL);
@@ -190,7 +188,6 @@ static void nl80211_appeared(const struct l_genl_family_info *info,
 	}
 }
 
-#ifdef HAVE_DBUS
 static void request_name_callback(struct l_dbus *dbus, bool success,
 					bool queued, void *user_data)
 {
@@ -261,7 +258,6 @@ static void dbus_disconnected(void *user_data)
 	l_info("D-Bus disconnected, quitting...");
 	iwd_shutdown();
 }
-#endif
 
 static void print_koption(const void *key, void *value, void *user_data)
 {
@@ -588,7 +584,6 @@ int main(int argc, char *argv[])
 	if (getenv("IWD_RTNL_DEBUG"))
 		l_netlink_set_debug(rtnl, do_debug, "[RTNL] ", NULL);
 
-#ifdef HAVE_DBUS
 	dbus = l_dbus_new_default(L_DBUS_SYSTEM_BUS);
 	if (!dbus) {
 		l_error("Failed to initialize D-Bus");
@@ -598,10 +593,6 @@ int main(int argc, char *argv[])
 	l_dbus_set_ready_handler(dbus, dbus_ready, dbus, NULL);
 	l_dbus_set_disconnect_handler(dbus, dbus_disconnected, NULL, NULL);
 	dbus_init(dbus);
-#else
-	l_genl_request_family(genl, NL80211_GENL_NAME, nl80211_appeared,
-				NULL, NULL);
-#endif
 
 	if (!setup_system_key())
 		goto failed_storage;
@@ -612,12 +603,8 @@ int main(int argc, char *argv[])
 	storage_exit();
 
 failed_storage:
-#ifdef HAVE_DBUS
 	dbus_exit();
 	l_dbus_destroy(dbus);
-#else
-(void)NULL;
-#endif
 
 failed_dbus:
 	l_netlink_destroy(rtnl);
