@@ -20,6 +20,7 @@
  *
  */
 
+struct auth_proto;
 struct sae_sm;
 struct handshake_state;
 
@@ -27,6 +28,10 @@ typedef void (*sae_tx_authenticate_func_t)(const uint8_t *data, size_t len,
 						void *user_data);
 typedef void (*sae_tx_associate_func_t)(void *user_data);
 
+bool sae_sm_is_h2e(struct auth_proto *ap);
+
+void sae_sm_set_force_group_19(struct auth_proto *ap);
+
 struct auth_proto *sae_sm_new(struct handshake_state *hs,
 				sae_tx_authenticate_func_t tx_auth,
 				sae_tx_associate_func_t tx_assoc,
